//! # Wooly Vault
//!
//! Wooly Vault is a set of smaller challenges for Rust hackers.
//!
//! A host may choose to open a challenge, set a secret and ask a contestant to try and solve the
//! challenge to get the secret. The contestant can then prove with the secret to the host that
//! they absolved the challenge.
//!
//! Wooly Vault has multiple challenges, and only one can be hosted per instance of this
//! application. The challenges all bind to a pre-set [Network Adress](crate::config::Config::addr).
//! The contestants can then connect to that adress with TCP. The first few challenges use raw TCP,
//! but some of the later ones are small HTTP webservices.
//!
//! Wooly Vault is programmed asynchronously with [tokio] to be able to handle many contestants at
//! once if needed.

use anyhow::anyhow;
use libpt::log::info;

use self::challenge::ChallengeLike;
use self::config::Config;
use self::vault::VaultRef;

pub mod challenge;
pub mod config;
#[cfg(feature = "meta-interface")]
pub mod meta;
pub mod vault;

#[inline]
pub(crate) async fn has_won(
    addr: &std::net::SocketAddr,
    vault: &VaultRef,
    contestant: &vault::Contestant,
) {
    vault.add_winner(contestant.clone()).await;
    libpt::log::info!("Sending the secret to {addr}")
}

/// Selects a single challenge by index and serves it with the given configuration and vault.
///
/// # Arguments
///
/// * `index` - The index of the challenge to select.
/// * `config` - The configuration for the challenge.
/// * `vault` - The vault that holds the secret for the challenge.
///
/// # Returns
///
/// A result indicating whether the challenge has successfully ended.
///
/// # Errors
///
/// Returns an error if no challenge with the given index exists, or if the challenge that is being
/// served errors.
pub async fn select_and_start_single(
    index: u16,
    config: Config,
    vault: VaultRef,
) -> anyhow::Result<()> {
    info!("select+start");
    match index {
        1 => challenge::c1::C1::new(config, vault).start().await?,
        2 => challenge::c2::C2::new(config, vault).start().await?,
        3 => challenge::c3::C3::new(config, vault).start().await?,
        _ => {
            return Err(anyhow!(
                "no challenge with index {index} does currently exist"
            ))
        }
    }
    Ok(())
}

pub async fn start_all(config: Config, vault: VaultRef) -> anyhow::Result<()> {
    let c1 = challenge::c1::C1::new(config.clone(), vault.clone());
    let c2 = challenge::c2::C2::new(config.clone(), vault.clone());
    let c3 = challenge::c3::C3::new(config.clone(), vault.clone());

    c1.start().await?;
    c2.start().await?;
    c3.start().await?;

    meta::serve(
        vec![
            challenge::c1::C1::text(),
            challenge::c2::C2::text(),
            challenge::c3::C3::text(),
        ],
        vault.clone(),
        config.clone(),
    )
    .await?;

    Ok(())
}
