use std::sync::Arc;

use anyhow::Result;
use libpt::log::tracing;
use minijinja::context;
use warp::reject::Rejection;
use warp::reply::Reply;
use warp::reply::Response;
use warp::Filter;

use super::errors::TemplateError;
use super::Service;

impl<'tp> Service<'tp> {
    pub fn admin_routes(
        this: Arc<Self>,
    ) -> impl Filter<Extract = impl Reply, Error = Rejection> + Clone + 'tp {
        let serv = this.clone();
        warp::path::end()
            .map(move || serv.clone())
            .and_then(overview)
            .with(warp::trace(|info| {
                // Create a span using tracing macros
                tracing::info_span!(
                    "admin",
                    method = %info.method(),
                    path = %info.path(),
                )
            }))
    }
}

async fn overview(serv: Arc<Service<'_>>) -> Result<Box<dyn warp::Reply>, warp::Rejection> {
    let challenge = serv.challenges[0].clone();
    let contestants = serv
        .vault
        .contestants()
        .await
        .into_iter()
        .collect::<Vec<_>>();
    let winners = serv.vault.winners().await.into_iter().collect::<Vec<_>>();
    let r = Response::new(
        serv.env
            .get_template("index")
            .map_err(TemplateError::from)?
            .render(context!(
                title => "Wooly-Vault",
                author => env!("CARGO_PKG_AUTHORS"),
                year => "2024",
                challenge_idx => serv.config.challenge,
                challenge_title => challenge.title(),
                challenge_description => challenge.description(),
                challenge_hints => challenge.hints(),
                challenge_solution => challenge.solution(),
                contestants => serv.vault.contestants().await.iter().collect::<Vec<_>>(),
                winners => serv.vault.winners().await.iter().collect::<Vec<_>>(),
                contestants => contestants,
                winners => winners,
                contestants_amount => contestants.len(),
                winners_amount => winners.len(),
            ))
            .map_err(TemplateError::from)?
            .into(),
    );

    Ok(Box::new(r))
}
