use std::sync::Arc;

use anyhow::Result;
use libpt::log::tracing;
use libpt::log::warn;
use minijinja::Environment;
use warp::reject::Rejection;
use warp::reply::Reply;
use warp::reply::Response;
use warp::Filter;

use crate::challenge::ChallengeDesc;
use crate::config::Config;
use crate::vault::VaultRef;

use self::errors::handle_rejection;

pub mod admin;
pub mod errors;

#[derive(Clone)]
pub struct Service<'tp> {
    vault: VaultRef,
    config: Config,
    env: Environment<'tp>,
    challenges: Vec<ChallengeDesc>,
}
impl<'tp> Service<'tp> {
    fn build(vault: VaultRef, config: Config, challenges: Vec<ChallengeDesc>) -> Result<Arc<Self>> {
        let mut env = Environment::new();
        env.add_template("index", include_str!("../../data/www/admin.html"))?;
        Ok(Self {
            vault,
            config,
            env,
            challenges,
        }
        .into())
    }
}

pub async fn serve(challenges: Vec<ChallengeDesc>, vault: VaultRef, config: Config) -> Result<()> {
    let service = Service::build(vault, config, challenges)?;

    let routes = Service::admin_routes(service.clone())
        .or(Service::ressources_routes())
        .recover(handle_rejection)
        .with(warp::trace(|info| {
            // Create a span using tracing macros
            tracing::info_span!(
                "admin",
                method = %info.method(),
                path = %info.path(),
            )
        }));

    warp::serve(routes)
        .run(service.config.addr_admin.unwrap())
        .await;

    warn!("exited the admin interface");
    Ok(())
}

impl Service<'_> {
    fn ressources_routes() -> impl Filter<Extract = impl Reply, Error = Rejection> + Clone {
        warp::path("styles.css").and_then(styles)
    }
}

async fn styles() -> Result<Box<dyn warp::Reply>, warp::Rejection> {
    let r = Response::new(include_str!("../../data/www/styles.css").to_string().into());
    Ok(Box::new(r))
}
